#ifndef lint
static char sccsid[] = "$Id: read_seg2_trace_data.c,v 1.9 2006/10/11 23:19:06 doug Exp $ ";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <strings.h>
#include <sys/file.h>
#include "seg2_struct.h"
#include "externals.h"
#include "procs.h"
#include "qlib2.h"

/*
#define NSCL_INDEX_NETWORK  0
#define NSCL_INDEX_STATION  1
#define NSCL_INDEX_CHANNEL  2
#define NSCL_INDEX_LOCATION 3
*/

#define DEBUGGING_FLOAT_DATA 1 /* CEDRIC DEBUGGING */
#define	DEBUGGING_MIN_MAX    1

/****************************************************************************
*  read_seg2_trace_data:						    *
*	Read SEG2 trace data.						    *
*  return:								    *
*	0 on success, 1 on failure.					    *
*	Set *hdr to point to DATA_HDR.					    *
*	Set *data to point to allocated data buffer.			    *
*	Caller must free DATA_HDR and data buffer.			    *
*  note:                                                                    *
*       It is necessary for phdr and pdata to be pointers to pointers,      *
*       in order to be able to assign to each of them the memory block      *
*       which has been allocated for it and its contents.  Here is what     *
*       happens if you try to pass in a pointer to struct instead:          *
*       Say the calling function has not allocated space for hdr, so it     *
*       points initially to NULL, and instead of passing to the function    *
*        DATA_HDR ** phdr                                                   *
*       you pass in                                                         *
*        DATA_HDR *hdr                                                      *
*       So the call to the function is                                      *
*        function(hdr);                                                     *
*       instead of                                                          *
*        function(&hdr);                                                    *
*       The called function gets the value of hdr, which is currently NULL, *
*       but interpreted as a pointer to NULL. When you assign to hdr        *
*        hdr = new_data_hdr();                                              *
*       the value of the pointer, hdr, local to the function , gets a new   *
*       value, the location of the allocated memory space, but the value    *
*       of the variable in the calling function is not adjusted.  That      *
*       is why the calling function needs to pass in the address of it's    *
*       variable, "function(&hdr)", so that the value stored in hdr's       *
*       real location can be updated.  That is why the parameter passed     *
*       is DATA_HDR **, because &(DATA_HDR * hdr) yields **.                *
****************************************************************************/
int read_seg2_trace_data
   (FILE	  *input,
    SEG2FILEDESC  *file_desc,
    SEG2TRACEDESC *trace_desc,
    DATA_HDR 	 **phdr,       /* See note above for why phdr and pdata are ptrs to ptr. */
    int		 **pdata)
{
  EXT_TIME   et;
  double     sample_interval;
  DATA_HDR  *hdr   = NULL;
  int       *data  = NULL;
  short int *sdata = NULL;
  float     *fdata = NULL;
  int        data_format, i;
  char      *val = NULL;       /* array to hold values from the trace_descriptor strings section */
  int        chars_in_value;   /* value is assigned by get_keyword_val_from_strings() */
  char       fn_status;        /* This is the status value returned by functions invoked */
/*NSCL      *nscl;*/           /* nscl[0], [1], [2], and [3] hold network, station, channel, and location, respectively. */
  NSCL       nscl;             /* nscl[0], [1], [2], and [3] hold network, station, channel, and location, respectively. */
  char      *stn;
  size_t (*read_chars)(void *, size_t, size_t, FILE *); /* function pointer */
  char       date[3][5];       /* holds date, indexed by mm, dd, and yy */
  const char dd = 0;
  const char mm = 1;
  const char yy = 2;
  char       time[3][3];       /* holds time, indexed by HH, MM, and SS */
  const char HH = 0;
  const char MM = 1;
  const char SS = 2;
  int	    status = 0;
  int	    column = 0;
  int	    iminval = 0;
  int	    imaxval = 0;
  float     dminval = 0;
  float     dmaxval = 0;
  int	    valid_date = 0;
  BLOCKETTE_1001 b1001;

  hdr = new_data_hdr();
  hdr->seq_no = 1;

  /***************************************************************
  * Set fields in DATA_HDR from SEG2 File and Trace Descriptors. *
  ***************************************************************/

  /* NETWORK, STATION, CHANNEL, AND LOCATION */
  if (determine_nscl(((trace_desc->known_keys)[CHANNEL_NUMBER])->val, hdr) != 0) {
    /* Either there was an error, or a mapping file indicated that *
     * this seg-2 channel should be skipped.                       *
     * Any error messages were already printed by determine_nscl() */
    hdr->station_id[0] = '\0';
    *phdr  = hdr;
    *pdata = data;
    return (4);
  }

  /* ACQUISITION DATE */
  if (strlen(acq_date) > 0) {
    /* An acquisition date value has been specified in the call to seg2toms *
     * (with the -D option) copy this value into the file descriptor's      *
     * known_keys section, and then clear the acq_date value, so that this  *
     * only happens once, even if there are many traces, since they all use *
     * the same date.  If the ACQUISITION_DATE keyword value pair does not  *
     * exist, it is created. */
    assign_new_acquisition_date(file_desc, acq_date);
    acq_date[0] = '\0';
  }
  valid_date = 0;
  if ((file_desc->known_keys)[ACQUISITION_DATE]) { /* this key & val pair exists */
    val = ((file_desc->known_keys)[ACQUISITION_DATE])->val;
    if (strlen(val) == 10) { /* presumably the 10 chars form the string "DD/MM/YYYY" */
      date[dd][0] = val[0];
      date[dd][1] = val[1];
      date[dd][2] = '\0';
      date[mm][0] = val[3];
      date[mm][1] = val[4];
      date[mm][2] = '\0';
      date[yy][0] = val[6];
      date[yy][1] = val[7];
      date[yy][2] = val[8];
      date[yy][3] = val[9];
      date[yy][4] = '\0';
      et.day   = atoi(date[dd]);
      et.month = atoi(date[mm]);
      et.year  = atoi(date[yy]);
      if (et.year > 0 && (et.month >= 1 && et.month <= 12) && et.day <= 31)
	valid_date = 1;
    }
    else if (strlen(val) == 11) { /* presumably the 10 chars form the string "DD/MMM/YYYY" */
      date[dd][0] = val[0];
      date[dd][1] = val[1];
      date[dd][2] = '\0';
      date[mm][0] = val[3];
      date[mm][1] = val[4];
      date[mm][2] = val[5];
      date[mm][3] = '\0';
      date[yy][0] = val[7];
      date[yy][1] = val[8];
      date[yy][2] = val[9];
      date[yy][3] = val[10];
      date[yy][4] = '\0';
      et.day   = atoi(date[dd]);
      et.month = month_index(date[mm]);
      et.year  = atoi(date[yy]);
      if (et.year > 0 && (et.month >= 1 && et.month <= 12) && et.day <= 31)
	valid_date = 1;
    }
    if (! valid_date) {
      fprintf(stderr, "Error: 'ACQUISITION_DATE' value of %d is invalid: Using 1/1/2000 as date.\n",
              val);
      et.year  = 2000;
      et.day   = 1;
      et.month = 1;
    }
  }
  else {
    fprintf(stderr, "Error: value of 'ACQUISITION_DATE' not found in file descriptor's strings: Using 1/1/2000 as date.\n");
    et.year  = 2000;
    et.day   = 1;
    et.month = 1;
  }
  et.doy = mdy_to_doy(et.month, et.day, et.year);

  /* ACQUISITION TIME: SECONDS FRACTION */
  if ((file_desc->known_keys)[ACQUISITION_SECOND_FRACTION]) { /* this key & val pair exists */
    /* In order to round the number, 0.5 is added to the float: *
     * if the fractional part was already >= .5, adding .5 will *
     * increase the integer part, thus rounding the number up.  *
     * Assigning float to int truncates the fractional part.    */
    et.usec = atof(((file_desc->known_keys)[ACQUISITION_SECOND_FRACTION])->val) * USECS_PER_SEC + .5;
  /*et.usec = atof(((file_desc->known_keys)[ACQUISITION_SECOND_FRACTION])->val) * USECS_PER_SEC;*/
  }
  else {
    et.usec = 0;
  }

  /* ACQUISITION TIME */
  if ((file_desc->known_keys)[ACQUISITION_TIME]) { /* this key & val pair exists */
    val = ((file_desc->known_keys)[ACQUISITION_TIME])->val;
    if (strlen(val) == 8) { /* presumably the 9 chars form the string "HH:MM:SS" */
      time[HH][0] = val[0];
      time[HH][1] = val[1];
      time[HH][2] = '\0';
      time[MM][0] = val[3];
      time[MM][1] = val[4];
      time[MM][2] = '\0';
      time[SS][0] = val[6];
      time[SS][1] = val[7];
      time[SS][2] = '\0';
      et.hour   = atoi(time[HH]);
      et.minute = atoi(time[MM]);
      et.second = atoi(time[SS]);
    }
    else {
      fprintf(stderr,
              "SEG-2 to MS error: length of 'ACQUISITION_TIME' value is not 8 chars ('HH:MM:SS') value is %s\nUsing 0:0:0 as time.\n",
              val);
      et.hour   = 0;
      et.minute = 0;
      et.second = 0;
    }
  }
  else {
   fprintf(stderr, "SEG-2 to MS error: value of 'ACQUISITION_TIME' not found in file descriptor's strings.\nUsing 0:0:0 as time.\n");
    et.hour   = 0;
    et.minute = 0;
    et.second = 0;
  }
  hdr->begtime = hdr->hdrtime = ext_to_int(et);
  
  /* Create a blockette 1001 for microsecond time resolution. */
  hdr->num_data_frames = (blksize/64) - 1;
  b1001.hdr.type = 1001;
  b1001.hdr.next = 0;
  b1001.clock_quality = clock_quality;
  b1001.usec99 = et.usec % 100;
  b1001.flags = 0;
  b1001.frame_count = hdr->num_data_frames;
  if (add_blockette (hdr, (void *)&b1001, 1001, sizeof(b1001), SEED_BIG_ENDIAN, -1) != 1) {
    fprintf (stderr, "Error creating blockette 1001\n");
    return (1);
  }

  /* SAMPLE_INTERVAL */
  if ((trace_desc->known_keys)[SAMPLE_INTERVAL]) { /* this key & val pair exists */
    sample_interval = atof(((trace_desc->known_keys)[SAMPLE_INTERVAL])->val);
    if (sample_interval == 0) {
      fprintf(stderr,
              "SEG-2 to MS error: value of 'SAMPLE_INTERVAL' is 0, can not use it to determine hdr->sample_rate.  Using value of 1\n");
      sample_interval = 1;
    }
  }
  else {
    fprintf(stderr, "SEG-2 to MS error: value of 'SAMPLE_INTERVAL' not found in file descriptor's strings.\nUsing value of 1.\n");
    sample_interval = 1;
  }
  if (sample_interval <= 1.0) {
    hdr->sample_rate = roundoff(1.0/sample_interval);
    hdr->sample_rate_mult = 1;
  }
  else {
    hdr->sample_rate = -1 * roundoff(sample_interval);
    hdr->sample_rate_mult = 1;
  }
  /* NUMBER OF SAMPLES */
  hdr->num_samples = trace_desc->numOfSamples;

  /* DATA FORMAT */
  switch (trace_desc->dataFormatCode) {
    case 1:   data_format = INT_16; break;
    case 2:   data_format = INT_32; break;
    case 3:   fprintf (stderr, "20-bit floating point (SEG-D) format unsupported.\n");         return(1);
    case 4:   data_format = IEEE_FP_SP; break; /* 32-bit IEEE Floating Point, Single Precision */
    case 5:   fprintf (stderr, "64-bit floating point (IEEE standard) format unsupported.\n"); return(1);
    default:  fprintf (stderr, "Invald data format code: %d\n", trace_desc->dataFormatCode);   return(1);
  }

    
  /*	Read SEG2 trace data.						*/
  data = (int *)malloc(sizeof(int) * hdr->num_samples);
  if (data == NULL) {
    fprintf (stderr, "Error mallocing data array for %d samples\n", hdr->num_samples);
    return (1);
  }

  /* Look at the SEG-2 File Descriptor's Block ID to              *
   * determine if the format of the file is my wordorder or not , *
   * and set the file character reading function accordingly.     */
  if      (file_desc->block_ID == NATIVE_WORDORDER_ID) { /* 0x3A55 */
    read_chars = fread; /* assign the function pointer */
  }
  else if (file_desc->block_ID == OPPOSITE_WORDORDER_ID) { /* 0x553A */
    read_chars = fread_swab; /* assign the function pointer */
  }
  else {
    fprintf(stderr,
            "Error: invalid SEG-2 File Descriptor Block ID: 0x%X. Skipping SEG-2 file.\n",
            file_desc->block_ID);
    return(1);
  }

  switch (data_format) {

    case INT_32:
      if (read_chars(data,
                     sizeof(int),
                     hdr->num_samples,
                     input) != hdr->num_samples) {
        fprintf (stderr, "Error reading %d samples of int data\n", hdr->num_samples);
        return(1);
      }
      if (dbgflag & DBG_FLAG_TD_MIN_MAX) {
	if (hdr->num_samples > 0) imaxval = iminval =  data[0];
	for (i=0; i<hdr->num_samples; i++) {
	    if (data[i] > imaxval) imaxval = data[i];
	    if (data[i] < iminval) iminval = data[i];
	}
	fprintf (info, "Trace min,max,range values are: %d %d %d\n", iminval, imaxval, imaxval-iminval);
	status = 4;
      }
      if (dbgflag & DBG_FLAG_TD_DUMP) {
	fprintf (info, "Integer trace data:\n");
	for (i=0; i<hdr->num_samples; i++) {
	    data[i] = (int)fdata[i];
	    fprintf (info, "%10i", data[i]);
	    column += 10;
	    if (column >= 80) {
		column = 0;
		fprintf (info, "\n");
	    }
	    else {
		fprintf (info, "  ");
	    }
	}
	fprintf (info, "\n");
	status = 4;
      }
      break;

    case INT_16:
      sdata = (short int *)malloc(sizeof(short int) * hdr->num_samples);
      if (sdata == NULL) {
        fprintf (stderr, "Error mallocing short int data array for %d samples\n", hdr->num_samples);
        return (1);
      }
      if (read_chars(sdata,
                     sizeof(short int),
                     hdr->num_samples,
                     input) != hdr->num_samples) {
        fprintf (stderr, "Error reading %d samples of short int data\n", hdr->num_samples);
        return(1);
      }
      /* Convert data to integer. */
      for (i=0; i<hdr->num_samples; i++) {
        data[i] = sdata[i];
      }
      free (sdata);
      if (dbgflag & DBG_FLAG_TD_MIN_MAX) {
	if (hdr->num_samples > 0) imaxval = iminval =  data[0];
	for (i=0; i<hdr->num_samples; i++) {
	    if (data[i] > imaxval) imaxval = data[i];
	    if (data[i] < iminval) iminval = data[i];
	}
	fprintf (info, "Trace min,max,range values are: %d %d %d\n", iminval, imaxval, imaxval-iminval);
	status = 4;
      }
      if (dbgflag & DBG_FLAG_TD_DUMP) {
	fprintf (info, "Integer trace data:\n");
	for (i=0; i<hdr->num_samples; i++) {
	    data[i] = (int)fdata[i];
	    fprintf (info, "%10i", data[i]);
	    column += 10;
	    if (column >= 80) {
		column = 0;
		fprintf (info, "\n");
	    }
	    else {
		fprintf (info, "  ");
	    }
	}
	fprintf (info, "\n");
	status = 4;
      }
      break;

    case IEEE_FP_SP:
      fdata = (float *)malloc(sizeof(float) * hdr->num_samples);
      if (fdata == NULL) {
        fprintf (stderr, "Error mallocing float data array for %d samples\n", hdr->num_samples);
        return (1);
      }
      if (read_chars(fdata,
                     sizeof(float),
                     hdr->num_samples,
                     input) != hdr->num_samples) {
        fprintf (stderr, "Error reading %d samples of float data\n", hdr->num_samples);
        return(1);
      }
      if (dbgflag & DBG_FLAG_TD_MIN_MAX) {
	if (hdr->num_samples > 0) dmaxval = dminval = fdata[0];
	for (i=0; i<hdr->num_samples; i++) {
	    if (fdata[i] > dmaxval) dmaxval = fdata[i];
	    if (fdata[i] < dminval) dminval = fdata[i];
	}
	fprintf (info, "Trace min,max,range values are: %.8lf %.8lf %.8lf\n", dminval, dmaxval, dmaxval-dminval);
	status = 4;
      }
      if (dbgflag & DBG_FLAG_TD_DUMP) {
	fprintf (info, "Real trace data:\n");
	for (i=0; i<hdr->num_samples; i++) {
	    data[i] = (int)fdata[i];
	    fprintf (info, "%13.8f", fdata[i]);
	    column += 13;
	    if (column >= 78) {
		column = 0;
		fprintf (info, "\n");
	    }
	}
	fprintf (info, "\n");
	status = 4;
      }
      if (! (dbgflag & (DBG_FLAG_TD_DUMP | DBG_FLAG_TD_MIN_MAX))) {
	/* Convert data to integer. */
	for (i=0; i<hdr->num_samples; i++) {
	    /* Adding +-.5 to the scaled float value rounds off the	*/
	    /* value correctly, rather than simply truncating it.	*/
	    if( fdata[i] >= 0.0 ) data[i] = (int)((float_scaling_factor*(double)fdata[i]) + 0.5000);
	    else                  data[i] = (int)((float_scaling_factor*(double)fdata[i]) - 0.5000);
	}
      }
      free (fdata);
      break;

    default:
      fprintf (stderr, "Unsupported data format: %d\n", data_format);
      return (1);
      break;
  }

  *phdr  = hdr;
  *pdata = data;

  return (status);
}


/*******************************************************************************
* Assign Network, Station, Channel (component), and Location (pod) values      *
* to the seed header, based on a default mapping, which can be over-ridden by  *
* specifying a mapping file and/or by explicitly specifying any single value   *
* for each of these fields in the call to seg2toms.  If a string is specified  *
* for any of Network, Station, Channel (component), or Location, it will       *
* overide both the default mapping and any mapping in a specified map file.    *
*  *
*******************************************************************************/
char determine_nscl(char val[], DATA_HDR *hdr) {
  int           i           = 0;
  unsigned char comp        = 0;
  unsigned char pod         = 0;
  char          returnval   = 0;
  int           channel_num = atoi(val);
  int           map_channel; /* int of seg-2 channel in map file, if it is used */

  /* Use explicit channel mapping if one is available. */
  if (map) {
    int channel_map_found = 0;
    while (map[i].seg2_channel >= 0 && ! channel_map_found) {
      if (channel_num == map[i].seg2_channel) {
	channel_map_found = 1;
        /* First check for an indication to skip a seg-2 channel */
	if ( (strcmp(map[i].seed_station,  "-") == 0) &&
             (strcmp(map[i].seed_network,  "-") == 0) &&
             (strcmp(map[i].seed_channel,  "-") == 0) &&
             (strcmp(map[i].seed_location, "-") == 0) ) {
          returnval |= 0x2;
        }
        else
        {
	    /* We always have a default station and network. */
	    strcpy(hdr->station_id, (strcmp(map[i].seed_station,"*") == 0) ? station : map[i].seed_station);
	    strcpy(hdr->network_id, (strcmp(map[i].seed_network,"*") == 0) ? network : map[i].seed_network);
	    /* However, we only have a default channel and location */
	    /* if they were specified on the command line. */
	    strcpy(hdr->channel_id, (strcmp(map[i].seed_channel,"*") == 0) ? channel : map[i].seed_channel);
	    if (hdr->channel_id[0] == '\0') {
		fprintf (info, "Error: no default SEED channel for seg2 channel %s\n", val);
	    }
	    strcpy(hdr->location_id, (strcmp(map[i].seed_location,"*") == 0) ? location : map[i].seed_location);
	    if (hdr->location_id[0] == '\0') {
		fprintf (info, "Error: no default SEED location for seg2 channel %s\n", val);
	    }
	}
      }
      i++;
    }
    /* If we did not find a mapping for this channel, it is an error. */
    if (! channel_map_found) {
	fprintf (info, "Error: channel map not found for seg2 channel %s\n", val);
	returnval |= 0x4;
    }
  }

  else {
    /* Use default channel mapping with optional command line override. */

    /* Start with a default mapping for channel (component) & location (pod): *
     * Each group of three consecutive Seg-2 channels are assigned to a       *
     * common pod number, which is then incremented for the next group.       *
     * Within each of those groups, the seg-2 channels are assigned to the    *
     * seed channel (components) as follows:                                  *
     *  the first  to GH1 (vertical:   Z),                                    *
     *  the second to GH2 (horizontal: N?), and                               *
     *  the third  to GH3 (horizontal: E?).                                   */
    pod  = (channel_num + 2) / 3;
    if ((0 < pod) && (pod <= 99))   sprintf(hdr->location_id, "%02d", pod);
    else {
	strcpy(hdr->location_id, "XX");
	returnval = 0x1;
	fprintf(stderr,
            "seg2toms error in determine_nscl(%s, hdr):\n"
            "   bad pod value %d for channel number %d. Using 'XX' as location.\n"
            "   Pod values should be 1-99.  Perhaps the seg-2 file has more than 300 channels,\n"
            "   since each pod value is by default assigned to a group of three consecutive channels.\n",
            val, pod, channel_num);
    }

    switch (channel_num % 3)
    {
	case 1: comp = 1; strcpy(hdr->channel_id, "GH1"); break; /* example: 4 % 3 = 1 -> comp 1 */
	case 2: comp = 2; strcpy(hdr->channel_id, "GH2"); break; /* example: 5 % 3 = 2 -> comp 2 */
	case 0: comp = 3; strcpy(hdr->channel_id, "GH3"); break; /* example: 6 % 3 = 0 -> comp 3 */
    }

    /* If S, N, C, or L parameters were specified on the command line, *
     *   set the seed header's corresponding field to the given        *
     *   station, network, channel, and/or location name.              *
     * Else, use default values (or those calculated above).           */
    strcpy(hdr->station_id, station);
    strcpy(hdr->network_id, network);
    if (strlen(channel)>0)  strcpy(hdr->channel_id, channel);
    if (strlen(location)>0) strcpy(hdr->location_id,location);

    if ((comp == 0) || (pod == 0)) {
	returnval |= 0x8;
	fprintf(stderr,
            "seg2toms error in determine_nscl(%s, hdr): invalid component (%d) or pod (%d) value for channel number %d.\n",
            val, comp, pod, channel_num);
    }
  }

  /* trim off any trailing blanks or newlines */
  trim (hdr->network_id);
  trim (hdr->station_id);
  trim (hdr->channel_id);
  trim (hdr->location_id);

  return returnval;
} 


void print_strings( SEG2FILEDESC  *file_desc, SEG2TRACEDESC *trace_desc ) {
  int i = 0;

  fprintf(info, "\n");
  fprintf(info, "The file descriptor keywords and values (from its strings section) are as follows:\n");
  for (i = 0; i < MAX_FD_KEY_INDEX; i++) {
    if (file_desc->known_keys[i])
      fprintf(info, "%30s = %s\n", file_desc->known_keys[i]->key, file_desc->known_keys[i]->val);
  }
  for (i = 0; i < file_desc->unknown_keys_count; i++) {
    fprintf(info, "%30s = %s\n", file_desc->unknown_keys[i]->key, file_desc->unknown_keys[i]->val);
  }

  fprintf(info, "The trace descriptor keywords and values (from its strings section) are as follows:\n");
  for (i = 0; i < MAX_TD_KEY_INDEX; i++) {
    if (trace_desc->known_keys[i])
      fprintf(info, "%30s = %s\n", trace_desc->known_keys[i]->key, trace_desc->known_keys[i]->val);
  }
  for (i = 0; i < trace_desc->unknown_keys_count; i++) {
    fprintf(info, "%30s = %s\n", trace_desc->unknown_keys[i]->key, trace_desc->unknown_keys[i]->val);
  }
}


char assign_new_acquisition_date(SEG2FILEDESC *file_desc, char *acq_date) {
  KEY_VAL_PAIR *key_val    = NULL;
  int           key_length = strlen("ACQUISITION_DATE");
  int           val_length = strlen(acq_date);

  if ((file_desc->known_keys)[ACQUISITION_DATE]) { /* this key & val pair exists */
    strcpy(((file_desc->known_keys)[ACQUISITION_DATE])->val, acq_date);
    ((file_desc->known_keys)[ACQUISITION_DATE])->val_length = val_length;
  }
  else {
    /* Allocate memory for the KEY_VAL_PAIR structure */
    key_val = (KEY_VAL_PAIR *)malloc(sizeof(KEY_VAL_PAIR));
    if (key_val == NULL) {
      fprintf(stderr,
              "read_seg2_trace_data: Can't allocate memory block for ACQUISITION_DATE key_val structure\n");
      return(1);
    } 
    /* Allocate memory for the KEY_VAL_PAIR structure's keyword array */
    key_val->key = (char *)malloc(sizeof(char) * (key_length + 1));
    if (key_val->key == NULL) {
      fprintf(stderr,
              "read_seg2_trace_data: Can't allocate memory block for ACQUISITION_DATE key_val's key array\n");
      free(key_val);
      return(1);
    }
    /* Allocate memory for the KEY_VAL_PAIR structure's value array */
    key_val->val = (char *)malloc(sizeof(char) * (val_length + 1));
    if (key_val->val == NULL) {
      fprintf(stderr,
              "read_seg2_trace_data: Can't allocate memory block for ACQUISITION_DATE key_val's val array\n");
      free(key_val->key);
      free(key_val);
      return(1);
    }
  
    /* Copy the keyword and value into the KEY_VAL_PAIR structure */
    strncpy(key_val->key, "ACQUISITION_DATE", key_length);
    strncpy(key_val->val, acq_date,           val_length);
    key_val->key[key_length] = '\0';
    key_val->val[val_length] = '\0';
    key_val->key_length = key_length;
    key_val->val_length = val_length;

    (file_desc->known_keys)[ACQUISITION_DATE] = key_val;
  }
  return (0);
}

static char *months[] = {
"JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC" };

int month_index(char *month)
{
    int i;
    for (i=0; i<12; i++) {
	if (strcasecmp (month,months[i]) == 0) return (i+1);
    }
    return (0);
}
	
